@@ -18,7 +18,6 @@
  */
 
 /*!
- * Copyright (c) 2019 by Contributors
  * \file lib_api.h
  * \brief APIs to interact with libraries
  * This API specifies function prototypes to
@@ -53,7 +52,7 @@
 #endif
 
 /* Make sure to update the version number everytime you make changes */
-#define MX_LIBRARY_VERSION 10
+#define MX_LIBRARY_VERSION 11
 
 /*!
  * \brief For loading multiple custom op libraries in Linux, exporting same symbol multiple
@@ -686,6 +685,18 @@ class CustomOpSelector {
  */
 class CustomStatefulOp {
  public:
+  CustomStatefulOp();
+  virtual ~CustomStatefulOp();
+
+  template<class A, typename ...Ts>
+  static CustomStatefulOp* create(Ts...args) {
+    CustomStatefulOp* op = new A(args...);
+    op->created = true;
+    return op;
+  }
+
+  bool wasCreated() { return created; }
+
   virtual MXReturnValue Forward(std::vector<MXTensor>* inputs,
                                 std::vector<MXTensor>* outputs,
                                 const OpResource& op_res) = 0;
@@ -695,15 +706,11 @@ class CustomStatefulOp {
     MX_ERROR_MSG << "Error! Operator does not support backward" << std::endl;
     return MX_FAIL;
   }
-};
 
-/*! \brief StatefulOp wrapper class to pass to backend OpState */
-class CustomStatefulOpWrapper {
- public:
-  explicit CustomStatefulOpWrapper(CustomStatefulOp* inst) : instance(inst) {}
-  CustomStatefulOp* get_instance() { return instance; }
+  bool ignore_warn;
+
  private:
-  CustomStatefulOp* instance;
+  bool created;
 };
 
 /*! \brief Custom Operator function templates */
@@ -911,13 +918,13 @@ class Registry {
 #define MX_TOSTRING(x) MX_STRINGIFY(x)
 
 /*! \brief declare a variable with custom name */
-#define MX_REGISTER_NAME_(Name) MXNet ## _CustomOp ## _
+#define MX_REGISTER_NAME_(Name) MXNet ## _CustomOp ## _ ## Name
 #define MX_REGISTER_DEF_(Name) mxnet::ext::CustomOp MX_REGISTER_NAME_(Name)
 
-#define MX_REGISTER_PROP_NAME_(Name) MXNet ## _CustomSubProp ## _
+#define MX_REGISTER_PROP_NAME_(Name) MXNet ## _CustomSubProp ## _ ## Name
 #define MX_REGISTER_PROP_DEF_(Name) mxnet::ext::CustomPartitioner MX_REGISTER_PROP_NAME_(Name)
 
-#define MX_REGISTER_PASS_NAME_(Name) MXNet ## _CustomPass ## _
+#define MX_REGISTER_PASS_NAME_(Name) MXNet ## _CustomPass ## _ ## Name
 #define MX_REGISTER_PASS_DEF_(Name) mxnet::ext::CustomPass MX_REGISTER_PASS_NAME_(Name)
 
 /*! \brief assign a var to a value */
@@ -1009,6 +1016,9 @@ typedef int (*opCallCreateOpState_t)(createOpState_t create_op, const char* cons
                                      int dev_id, unsigned int** inshapes, int* indims,
                                      int num_in, const int* intypes, void** state_op);
 
+#define MXLIB_OPCALLDESTROYOPSTATE_STR "_opCallDestroyOpState"
+typedef int (*opCallDestroyOpState_t)(void* state_op);
+
 #define MXLIB_OPCALLFSTATEFULCOMP_STR "_opCallFStatefulCompute"
 typedef int (*opCallFStatefulComp_t)(int is_forward, void* state_op,
                                      const int64_t** inshapes, int* indims,
@@ -1118,6 +1128,18 @@ typedef int (*msgSize_t)(void);
 #define MXLIB_MSGGET_STR "_msgGet"
 typedef int (*msgGet_t)(int idx, const char** msg);
 
+/*! \brief StatefulOp wrapper class to pass to backend OpState */
+class CustomStatefulOpWrapper {
+ public:
+  ~CustomStatefulOpWrapper();
+  explicit CustomStatefulOpWrapper(CustomStatefulOp* inst, opCallDestroyOpState_t destroy)
+    : instance(inst), destroy_(destroy) {}
+  CustomStatefulOp* get_instance() { return instance; }
+ private:
+  CustomStatefulOp* instance;
+  opCallDestroyOpState_t destroy_;
+};
+
 #if defined(_WIN32) || defined(_WIN64) || defined(__WINDOWS__)
 #define MX_INT_RET  __declspec(dllexport) int __cdecl
 #define MX_VOID_RET __declspec(dllexport) void __cdecl
@@ -1200,6 +1222,9 @@ extern "C" {
                                   int dev_id, unsigned int** inshapes, int* indims,
                                   int num_in, const int* intypes, void** state_op);
 
+  /*! \brief returns status of deleting StatefulOp instance for operator from library */
+  MX_VOID_RET _opCallDestroyOpState(void* state_op);
+
   /*! \brief returns status of calling Stateful Forward/Backward for operator from library */
   MX_INT_RET _opCallFStatefulCompute(int is_forward, void* state_op, const int64_t** inshapes,
                                      int* indims, void** indata, int* intypes, size_t* inIDs,
