@@ -18,7 +18,6 @@
  */
 
 /*!
- * Copyright (c) 2019 by Contributors
  * \file lib_api.cc
  * \brief APIs to interact with libraries
  * This API specifies function prototypes to
@@ -858,6 +857,13 @@ void mxnet::ext::CustomOp::raiseDuplicateContextError() {
     + op_name_str + "'");
 }
 
+mxnet::ext::CustomStatefulOp::CustomStatefulOp() : ignore_warn(false), created(false) {}
+mxnet::ext::CustomStatefulOp::~CustomStatefulOp() {}
+
+mxnet::ext::CustomStatefulOpWrapper::~CustomStatefulOpWrapper() {
+  destroy_(instance);
+}
+
 mxnet::ext::CustomPass::CustomPass() : name("ERROR") {}
 mxnet::ext::CustomPass::CustomPass(const char* pass_name)
   : name(pass_name) {}
@@ -1248,6 +1254,13 @@ MX_INT_RET _opCallCreateOpState(mxnet::ext::createOpState_t create_op, const cha
   return create_op(attrs, ctx, in_shapes, in_types, op_ptr);
 }
 
+/*! \brief calls StatefulOp destructor for operator from library */
+MX_VOID_RET _opCallDestroyOpState(void* state_op) {
+  mxnet::ext::CustomStatefulOp* op_ptr =
+    reinterpret_cast<mxnet::ext::CustomStatefulOp*>(state_op);
+  delete op_ptr;
+}
+
 /*! \brief returns status of calling Stateful Forward/Backward for operator from library */
 MX_INT_RET _opCallFStatefulCompute(int is_forward, void* state_op, const int64_t** inshapes,
                                    int* indims, void** indata, int* intypes, size_t* inIDs,
